#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "nixi.h"
#include <util/delay.h>

void set_digit(uint8_t *data, uint8_t bitnumber) {
    uint8_t byte, bit;

    byte = bitnumber / 8;
    bit  = bitnumber % 8;

    nixi[byte] |= (1<<bit);
}

void clear_digit(uint8_t digit) {
    uint8_t byte, bit, i, bitnumber;

    for (i=0; i<10; i++) {
        bitnumber = pgm_read_byte(&digits[digit][i]);
        byte = bitnumber / 8;
        bit  = bitnumber % 8;
        nixi[byte] &= ~(1<<bit);
    }
}

void clear_all(void) {
    uint8_t i;

    for (i=0; i<sizeof(nixi); i++) {
        nixi[i] = 0;
    }
}


void update_nixi(uint8_t *data) {
    int8_t i;

    NCS_SER_LOW
    for (i=8; i>=0; i--) {
        SPDR=nixi[i];
        while( !(SPSR & (1<<SPIF)) );
    }
    NCS_SER_HIGH
}

void make_time(uint8_t *nixiarray) {

    uint8_t i, h,m,s;
    uint32_t time_local;

    // clear nixi data

    for (i=0; i<9; i++) {
        nixiarray[i]=0;
    }
    
    // get time, atomic

    cli();
    time_local = time;
    sei();

    // convert to h:m:s 

    s = time_local % 60;
    time_local /= 60;
    m = time_local % 60;
    time_local /= 60;
    h = time_local;

    // convert time to nixi
    // set bits

    set_digit(nixiarray, pgm_read_byte(&digits[5][s%10]));
    set_digit(nixiarray, pgm_read_byte(&digits[4][s/10]));
    set_digit(nixiarray, pgm_read_byte(&digits[3][m%10]));
    set_digit(nixiarray, pgm_read_byte(&digits[2][m/10]));
    set_digit(nixiarray, pgm_read_byte(&digits[1][h%10]));
    set_digit(nixiarray, pgm_read_byte(&digits[0][h/10]));

    set_digit(nixi, pgm_read_byte(&digits[6][0]));      // DOT 1
    set_digit(nixi, pgm_read_byte(&digits[6][1]));      // DOT 2
}

